/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2008 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    simpleScalarFoam

Description
    Steady-state solver for incompressible, laminar flow, with scalar transport

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"
#   include "createTime.H"
#   include "createMesh.H"
#   include "createFields.H"
#   include "initContinuityErrs.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nStarting time loop\n" << endl;

    for (runTime++; !runTime.end(); runTime++)
    {
        Info<< "Time = " << runTime.timeName() << nl << endl;

#       include "readSIMPLEControls.H"
#       include "initConvergenceCheck.H"

        p.storePrevIter();

        // Pressure-velocity SIMPLE corrector
        {
#           include "UEqn.H"
#           include "pEqn.H"
        }

// Solves scalar equation
        T.storePrevIter();

        for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
        {
            fvScalarMatrix TEqn
            (
                fvm::div(phi, T)
              - fvm::laplacian(DT, T)
            );

            TEqn.relax();

            TEqn.solve();
	}

// Calculates kc and Sh on each patch
         Info<< "Calculating kc and Sh" << endl;

         volScalarField kc
         (
                IOobject
                (
                    "kc",
                    runTime.timeName(),
                    mesh,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh,
                dimensionedScalar
                (
                    "kc",
                    DT.dimensions()/dimLength,
                    0
                )
          );

	  forAll(kc.boundaryField(), patchi)
	  {
//          	  kc = 1 / Tb * D * (dT/dy)_wall
		  kc.boundaryField()[patchi] =
                    1/Tb.value()*DT.value()*-T.boundaryField()[patchi].snGrad();

// // 		Calculates average kc
// 	    if
// 	      (
// 		  (mesh.boundaryMesh()[patchi].name() == "electrode1")
// 	      ) 
//               {  
// 		scalar area = gSum(mesh.magSf().boundaryField()[patchi]);
//                 scalar sumField = 0;
// 
//                 if (area > 0)
//                 {
//                     sumField = gSum
//                     (
//                         mesh.magSf().boundaryField()[patchi]
//                       * kc.boundaryField()[patchi]
//                     ) / area;
//                 }
// 
//                 Info<< "    Average of kc over patch "
//                     << patchi << " = "
//                     << sumField << endl;
// 
// 	      }
	  }

           volScalarField Sh
           (
                IOobject
                (
                    "Sh",
                    runTime.timeName(),
                    mesh,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
// 		Sh = kc * d / DT
		kc*d/DT
           );

        runTime.write();

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;

#       include "convergenceCheck.H"

        }

    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
